import streamlit as st
from ApiClient import ApiClient
from auth_libs.Users import check_auth_status
from components.agent_selector import agent_selector

check_auth_status()

agent_name = agent_selector()
st.title("Manage Tasks")


if agent_name:
    smart_task_toggle = st.checkbox("Enable Smart Task")
    task_objective = st.text_area("Enter the task objective")
    existing_tasks = ApiClient.get_tasks(agent_name=agent_name)
    status = ApiClient.get_task_status(agent_name=agent_name)
    agent_status = "Not Running" if status == False else "Running"
    load_task = st.selectbox(
        "Load Task",
        options=[""] + existing_tasks,
        index=0,
    )

    col1, col2 = st.columns([3, 1])
    with col1:
        columns = st.columns([3, 2])
        if st.button("Start Task", key=f"start_{agent_name}"):
            if agent_name and (task_objective or load_task):
                ApiClient.start_task_agent(
                    agent_name=agent_name, objective=task_objective
                )
                st.experimental_rerun()
            else:
                columns[0].error("Agent name and task objective are required.")

        if st.button("Stop Task", key=f"stop_{agent_name}"):
            # This actually toggles to stop it if you try to run while it is running.
            ApiClient.start_task_agent(agent_name=agent_name, objective="")
            st.experimental_rerun()

    with col2:
        st.markdown(f"**Status:** {agent_status}")



import requests
import random


class LlamacppapiProvider:
    def __init__(
        self,
        AI_PROVIDER_URI: str = "http://localhost:8000",
        MAX_TOKENS: int = 2048,
        AI_TEMPERATURE: float = 0.7,
        AI_MODEL: str = "default",
        STOP_SEQUENCE: str = "</s>",
        **kwargs,
    ):
        self.AI_PROVIDER_URI = AI_PROVIDER_URI
        self.AI_TEMPERATURE = AI_TEMPERATURE
        self.MAX_TOKENS = MAX_TOKENS
        self.AI_MODEL = AI_MODEL
        self.STOP_SEQUENCE = STOP_SEQUENCE
        self.MAX_TOKENS = int(self.MAX_TOKENS)

    async def instruct(self, prompt, tokens: int = 0):
        params = {
            "prompt": prompt,
            "temperature": float(self.AI_TEMPERATURE),
            "stop": self.STOP_SEQUENCE,
            "seed": random.randint(1, 1000000000),
        }
        response = requests.post(f"{self.AI_PROVIDER_URI}/v1/completion", json=params)
        data = response.json()
        return data["content"]



import streamlit as st
from ApiClient import ApiClient
import logging
from auth_libs.Users import check_auth_status

check_auth_status()


st.header("Manage Chains")
st.markdown("### Predefined Injection Variables")
st.markdown(
    """
    Any of these variables can be used in command arguments or prompt arguments to inject data into the prompt. These can also be used inside of any Custom Prompt.
- `{agent_name}` will cause the agent name to be injected.
- `{context}` will cause the current context from memory to be injected. (Only applies to prompts but is still a reserved variable name.)
- `{date}` will cause the current date and timestamp to be injected.
- `{COMMANDS}` will cause the available commands list to be injected and for automatic commands execution from the agent based on its suggestions.
- `{command_list}` will cause the available commands list to be injected, but will not execute any commands the AI chooses. Useful on validation steps.
- `{STEPx}` will cause the step `x` response from a chain to be injected. For example, `{STEP1}` will inject the first step's response in a chain.
"""
)
chain_names = ApiClient.get_chains()
agents = ApiClient.get_agents()
chain_action = st.selectbox("Action", ["Create Chain", "Delete Chain", "Run Chain"])
if chain_action == "Create Chain":
    chain_name = st.text_input("Chain Name")
else:
    chain_name = st.selectbox("Chains", chain_names)

if st.button("Perform Action"):
    if chain_name:
        if chain_action == "Create Chain":
            ApiClient.add_chain(chain_name=chain_name)
            st.success(f"Chain '{chain_name}' created.")
            st.experimental_rerun()
        elif chain_action == "Delete Chain":
            ApiClient.delete_chain(chain_name=chain_name)
            st.success(f"Chain '{chain_name}' deleted.")
            st.experimental_rerun()
        elif chain_action == "Run Chain":
            ApiClient.run_chain(chain_name=chain_name)
            st.success(f"Chain '{chain_name}' executed.")
    else:
        st.error("Chain name is required.")

st.header("Manage Chain Steps & View Responses")


selected_chain_name = st.selectbox("Select Chain", [""] + chain_names)

if selected_chain_name:
    try:
        chain = ApiClient.get_chain(chain_name=selected_chain_name)
    except:
        st.write(selected_chain_name + " Responses: ")
        try:
            if "_responses" in selected_chain_name:
                chain_commands_executed = ApiClient.get_chain_responses(
                    chain_name=selected_chain_name.replace("_responses", "")
                )
            else:
                chain_commands_executed = False
            chain_response = ApiClient.get_chain_responses(
                chain_name=selected_chain_name,
            )
            if chain_response and chain_commands_executed:
                for exec in chain_commands_executed["steps"]:
                    logging.info("----------exec: " + str(exec["step"]))
                    logging.info("----------Chain_Response " + str(chain_response["1"]))
                    st.write(exec)
                    st.write(chain_response[str(exec["step"])])
            elif chain_response:
                st.write(chain_response)
            else:
                raise ValueError("End of responses!", "None Found!")
        except ValueError as err:
            st.write(err.args)
            loop = False
        st.stop()

    st.subheader(f"Selected Chain: {selected_chain_name}")

    def modify_step(step):
        step_number = step["step"]
        agent_name = step["agent_name"]
        prompt_type = step["prompt_type"]
        prompt = step.get("prompt", "")
        modify_step_number = st.number_input(
            "Step Number",
            min_value=1,
            step=1,
            value=step_number,
            key=f"step_num_{step_number}",
        )
        modify_agent_name = st.selectbox(
            "Select Agent",
            options=[""] + [agent["name"] for agent in agents],
            index=([agent["name"] for agent in agents].index(agent_name) + 1)
            if agent_name in [agent["name"] for agent in agents]
            else 0,
            key=f"agent_name_{step_number}",
        )
        modify_prompt_type = st.selectbox(
            "Select Prompt Type",
            options=["", "Command", "Prompt"],
            index=["", "Command", "Prompt"].index(prompt_type),
            key=f"prompt_type_{step_number}",
        )

        if modify_prompt_type == "Command":
            available_commands = [
                cmd["friendly_name"]
                for cmd in ApiClient.get_commands(agent_name=modify_agent_name)
            ]
            command_name = st.selectbox(
                "Select Command",
                [""] + available_commands,
                key=f"command_name_{step_number}",
                index=available_commands.index(prompt.get("command_name", "")) + 1
                if "command_name" in prompt
                else 0,
            )

            if command_name:
                command_args = ApiClient.get_command_args(command_name)
                formatted_command_args = ", ".join(
                    [
                        f"{arg}: {st.text_input(arg, value=prompt.get(arg, ''), key=f'{arg}_{step_number}')} "
                        for arg in command_args
                        if arg != "context"
                        and arg != "command_list"
                        and arg != "COMMANDS"
                    ]
                )
                modify_prompt = f"{command_name}({formatted_command_args})"
        elif modify_prompt_type == "Prompt":
            available_prompts = ApiClient.get_prompts()
            modify_prompt_name = st.selectbox(
                "Select Custom Prompt",
                [""] + available_prompts,
                key=f"prompt_name_{step_number}",
                index=available_prompts.index(prompt.get("prompt_name", "")) + 1
                if "prompt_name" in prompt
                else 0,
            )

            if modify_prompt_name:
                prompt_args = ApiClient.get_prompt_args(modify_prompt_name)
                if prompt_args:
                    if isinstance(prompt_args, str):
                        prompt_args = [prompt_args]
                    try:
                        modify_prompt["prompt_name"] = modify_prompt_name
                        for arg in prompt_args:
                            if (
                                arg != "context"
                                and arg != "command_list"
                                and arg != "COMMANDS"
                            ):
                                modify_prompt[arg] = st.text_input(
                                    arg,
                                    value=prompt.get(arg, "") if arg in prompt else "",
                                    key=f"{arg}_{step_number}",
                                )
                    except:
                        pass
        else:
            modify_prompt = ""

        if st.button("Modify Step", key=f"modify_{step_number}"):
            modify_prompt = {}
            if modify_prompt_type == "Command":
                modify_prompt["command_name"] = command_name
                for arg in command_args:
                    if arg != "context" and arg != "command_list" and arg != "COMMANDS":
                        modify_prompt[arg] = st.session_state[f"{arg}_{step_number}"]
            elif modify_prompt_type == "Prompt":
                modify_prompt["prompt_name"] = modify_prompt_name
                for arg in prompt_args:
                    if arg != "context" and arg != "command_list" and arg != "COMMANDS":
                        modify_prompt[arg] = st.session_state[f"{arg}_{step_number}"]

            ApiClient.update_step(
                chain_name=selected_chain_name,
                step_number=step_number,
                agent_name=modify_agent_name,
                prompt_type=modify_prompt_type,
                prompt=modify_prompt,
            )
            st.success(f"Step {step_number} updated in chain '{selected_chain_name}'.")
            st.experimental_rerun()
        return step

    st.write("Existing Steps:")
    if chain:
        for step in chain:
            if step is not None:
                try:
                    step = modify_step(step)
                except TypeError:
                    st.error(
                        "Error loading chain step. Please check the chain configuration."
                    )

    step_number = max([s["step"] for s in chain]) + 1 if chain else 1
    agent_name = st.selectbox(
        "Select Agent",
        options=[""] + [agent["name"] for agent in agents],
        index=0,
        key="add_step_agent_name",
    )
    prompt_type = st.selectbox(
        "Select Prompt Type",
        [""] + ["Command", "Prompt"],
        key="add_step_prompt_type",
    )
    agent_commands = ApiClient.get_commands(agent_name=agent_name)
    if prompt_type == "Command":
        available_commands = [cmd["friendly_name"] for cmd in agent_commands]
        command_name = st.selectbox(
            "Select Command",
            [""] + available_commands,
            key="add_step_command_name",
        )

        if command_name:
            command_args = ApiClient.get_command_args(command_name=command_name)
            formatted_command_args = ", ".join(
                [
                    f"{arg}: {st.text_input(arg, key=f'add_step_{arg}')} "
                    for arg in command_args
                    if arg != "context" and arg != "command_list" and arg != "COMMANDS"
                ]
            )
            prompt = f"{command_name}({formatted_command_args})"
    elif prompt_type == "Prompt":
        available_prompts = ApiClient.get_prompts()
        prompt_name = st.selectbox(
            "Select Custom Prompt",
            [""] + available_prompts,
            key="add_step_prompt_name",
        )

        if prompt_name:
            prompt_args = ApiClient.get_prompt_args(prompt_name)
            formatted_prompt_args = ", ".join(
                [
                    f"{arg}: {st.text_input(arg, key=f'add_step_{arg}')} "
                    for arg in prompt_args
                    if arg != "context" and arg != "command_list" and arg != "COMMANDS"
                ]
            )
            prompt = f"{prompt_name}({formatted_prompt_args})"
    else:
        prompt = ""

    step_action = st.selectbox(
        "Action",
        ["Add Step", "Update Step", "Delete Step"],
        key="add_step_action",
    )

    if st.button("Perform Step Action", key="add_step_button"):
        if (
            selected_chain_name
            and step_number
            and agent_name
            and prompt_type
            and prompt
        ):
            prompt_data = {}
            if prompt_type == "Command":
                prompt_data["command_name"] = command_name
                for arg in command_args:
                    if arg != "context" and arg != "command_list" and arg != "COMMANDS":
                        prompt_data[arg] = st.session_state[f"add_step_{arg}"]
            elif prompt_type == "Prompt":
                prompt_data["prompt_name"] = prompt_name
                for arg in prompt_args:
                    if arg != "context" and arg != "command_list" and arg != "COMMANDS":
                        prompt_data[arg] = st.session_state[f"add_step_{arg}"]
            elif step_action == "Update Step":
                if prompt_type == "Command":
                    prompt_data = {"command_name": command_name}
                    for arg in command_args:
                        if (
                            arg != "context"
                            and arg != "command_list"
                            and arg != "COMMANDS"
                        ):
                            prompt_data[arg] = st.session_state[f"add_step_{arg}"]
                elif prompt_type == "Prompt":
                    prompt_data = {"prompt_name": prompt_name}
                    for arg in prompt_args:
                        if (
                            arg != "context"
                            and arg != "command_list"
                            and arg != "COMMANDS"
                        ):
                            prompt_data[arg] = st.session_state[f"add_step_{arg}"]

                ApiClient.update_step(
                    chain_name=selected_chain_name,
                    step_number=step_number,
                    agent_name=agent_name,
                    prompt_type=prompt_type,
                    prompt=prompt_data,
                )
                st.success(
                    f"Step {step_number} updated in chain '{selected_chain_name}'."
                )
                st.experimental_rerun()
            elif step_action == "Delete Step":
                ApiClient.delete_step(selected_chain_name, step_number)
                st.success(
                    f"Step {step_number} deleted from chain '{selected_chain_name}'."
                )
                st.experimental_rerun()
        else:
            st.error("All fields are required.")
else:
    st.warning("Please select a chain to manage steps.")



import streamlit as st
from streamlit import (
    markdown,
    header,
    checkbox,
    container,
    text_input,
    button,
    spinner,
    error,
    warning,
)
from auth_libs.Users import check_auth_status
from components.agent_selector import agent_selector
from ApiClient import ApiClient

check_auth_status()
agent_name = agent_selector()


def render_history(instruct_container, chat_history):
    instruct_container.empty()
    with instruct_container:
        for chat in chat_history:
            if "role" in chat and "message" in chat:
                markdown(
                    f'<div style="text-align: left; margin-bottom: 5px;"><strong>{chat["role"]}:</strong> {chat["message"]}</div>',
                    unsafe_allow_html=True,
                )


header("Instruct an Agent")

smart_instruct_toggle = checkbox("Enable Smart Instruct")

st.session_state["chat_history"] = {}

instruct_container = container()

if agent_name:
    try:
        st.session_state["chat_history"][agent_name] = ApiClient.get_chat_history(
            agent_name=agent_name
        )
    except:
        st.session_state["chat_history"][
            agent_name
        ] = []  # initialize as an empty list, not a dictionary

    render_history(
        instruct_container=instruct_container,
        chat_history=st.session_state["chat_history"][agent_name],
    )
    instruct_prompt = text_input("Enter your message", key="instruct_prompt")
    send_button = button("Send Message")

    if send_button:
        if agent_name and instruct_prompt:
            with spinner("Thinking, please wait..."):
                if smart_instruct_toggle:
                    response = ApiClient.smartinstruct(
                        agent_name=agent_name,
                        prompt=instruct_prompt,
                        shots=3,
                    )
                else:
                    response = ApiClient.instruct(
                        agent_name=agent_name,
                        prompt=instruct_prompt,
                    )

            instruct_entry = [
                {"role": "USER", "message": instruct_prompt},
                {"role": agent_name, "message": response},
            ]
            st.session_state["chat_history"][agent_name].extend(instruct_entry)
            render_history(
                instruct_container=instruct_container,
                chat_history=st.session_state["chat_history"][agent_name],
            )
        else:
            error("Agent name and message are required.")
else:
    warning("Please select an agent to start instructting.")



import uvicorn
from fastapi import FastAPI, HTTPException
from fastapi.middleware.cors import CORSMiddleware
from pydantic import BaseModel
from Config import Config
from AGiXT import AGiXT
from Agent import Agent
from Chain import Chain
from Tasks import Tasks
from Prompts import Prompts
from typing import Optional, Dict, List, Any
from provider import get_provider_options
from Embedding import get_embedding_providers
from Extensions import Extensions
import os
import logging
import argparse
import asyncio

CFG = Config()
app = FastAPI(
    title="AGiXT",
    description="AGiXT is an Artificial Intelligence Automation platform for creating and managing AI agents. Visit the GitHub repo for more information or to report issues. https://github.com/Josh-XT/AGiXT/",
    version="1.0.0",  # API version according to https://restfulapi.net/versioning/
    docs_url="/",
)
agent_threads = {}
agent_stop_events = {}

app.add_middleware(
    CORSMiddleware,
    allow_origins=["*"],
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
)


class AgentName(BaseModel):
    agent_name: str


class AgentNewName(BaseModel):
    new_name: str


class Objective(BaseModel):
    objective: str


class Prompt(BaseModel):
    prompt: str


class PromptName(BaseModel):
    prompt_name: str


class PromptList(BaseModel):
    prompts: List[str]


class ChainNewName(BaseModel):
    new_name: str


class ChainName(BaseModel):
    chain_name: str


class StepInfo(BaseModel):
    step_number: int
    agent_name: str
    prompt_type: str
    prompt: dict


class ChainStep(BaseModel):
    step_number: int
    agent_name: str
    prompt_type: str
    prompt: dict


class ChainStepNewInfo(BaseModel):
    old_step_number: int
    new_step_number: int


class ResponseMessage(BaseModel):
    message: str


class UrlInput(BaseModel):
    url: str


class FileInput(BaseModel):
    file_name: str
    file_content: str


class TaskOutput(BaseModel):
    output: str
    message: Optional[str] = None


class ToggleCommandPayload(BaseModel):
    command_name: str
    enable: bool


class CustomPromptModel(BaseModel):
    prompt_name: str
    prompt: str


class AgentSettings(BaseModel):
    agent_name: str
    settings: Dict[str, Any]


class AgentCommands(BaseModel):
    agent_name: str
    commands: Dict[str, Any]


@app.get("/api/provider", tags=["Provider"])
async def get_providers():
    providers = CFG.get_providers()
    return {"providers": providers}


@app.get("/api/provider/{provider_name}", tags=["Provider"])
async def get_provider_settings(provider_name: str):
    settings = get_provider_options(provider_name=provider_name)
    return {"settings": settings}


@app.get("/api/embedding_providers", tags=["Provider"])
async def get_embed_providers():
    providers = get_embedding_providers()
    return {"providers": providers}


@app.post("/api/agent", tags=["Agent"])
async def add_agent(agent: AgentSettings) -> Dict[str, str]:
    agent_info = Agent(agent.agent_name).add_agent(
        agent_name=agent.agent_name, provider_settings=agent.settings
    )
    return {"message": "Agent added", "agent_file": agent_info["agent_file"]}


@app.patch("/api/agent/{agent_name}", tags=["Agent"])
async def rename_agent(agent_name: str, new_name: AgentNewName) -> ResponseMessage:
    Agent(agent_name=agent_name).rename_agent(
        agent_name=agent_name, new_name=new_name.new_name
    )
    return ResponseMessage(
        message=f"Agent {agent_name} renamed to {new_name.new_name}."
    )


@app.put("/api/agent/{agent_name}", tags=["Agent"])
async def update_agent_settings(
    agent_name: str, settings: AgentSettings
) -> ResponseMessage:
    update_config = Agent(agent_name=agent_name).update_agent_config(
        new_config=settings.settings, config_key="settings"
    )
    return ResponseMessage(message=update_config)


@app.post("/api/agent/{agent_name}/learn/file", tags=["Agent"])
async def learn_file(agent_name: str, file: FileInput) -> ResponseMessage:
    file_path = os.path.join(os.getcwd(), file.file_name)
    with open(file_path, "w") as f:
        f.write(file.file_content)
    try:
        memories = Agent(agent_name=agent_name).get_memories()
        await memories.mem_read_file(file_path=file.file_content)
        try:
            os.remove(file_path)
        except Exception:
            pass
        return ResponseMessage(message="Agent learned the content from the file.")
    except Exception as e:
        try:
            os.remove(file_path)
        except Exception:
            pass
        raise HTTPException(status_code=500, detail=str(e))


@app.post("/api/agent/{agent_name}/learn/url", tags=["Agent"])
async def learn_url(agent_name: str, url: UrlInput) -> ResponseMessage:
    try:
        memories = Agent(agent_name=agent_name).get_memories()
        await memories.read_website(url=url.url)
        return ResponseMessage(message="Agent learned the content from the url.")
    except Exception as e:
        raise HTTPException(status_code=500, detail=str(e))


@app.put("/api/agent/{agent_name}/commands", tags=["Agent"])
async def update_agent_commands(
    agent_name: str, commands: AgentCommands
) -> ResponseMessage:
    update_config = Agent(agent_name=agent_name).update_agent_config(
        new_config=commands.commands, config_key="commands"
    )
    return ResponseMessage(message=update_config)


@app.delete("/api/agent/{agent_name}", tags=["Agent"])
async def delete_agent(agent_name: str) -> ResponseMessage:
    result, status_code = Agent(agent_name=agent_name).delete_agent(
        agent_name=agent_name
    )
    if status_code == 200:
        return ResponseMessage(message=result["message"])
    else:
        raise HTTPException(status_code=status_code, detail=result["message"])


@app.get("/api/agent", tags=["Agent"])
async def get_agents():
    agents = CFG.get_agents()
    return {"agents": agents}


@app.get("/api/agent/{agent_name}", tags=["Agent"])
async def get_agentconfig(agent_name: str):
    agent_config = Agent(agent_name=agent_name).get_agent_config()
    return {"agent": agent_config}


@app.get("/api/{agent_name}/chat", tags=["Agent"])
async def get_chat_history(agent_name: str):
    chat_history = Agent(agent_name=agent_name).get_chat_history(agent_name=agent_name)
    return {"chat_history": chat_history}


@app.delete("/api/agent/{agent_name}/memory", tags=["Agent"])
async def wipe_agent_memories(agent_name: str) -> ResponseMessage:
    Agent(agent_name=agent_name).wipe_agent_memories(agent_name=agent_name)
    return ResponseMessage(message=f"Memories for agent {agent_name} deleted.")


@app.post("/api/agent/{agent_name}/instruct", tags=["Agent"])
async def instruct(agent_name: str, prompt: Prompt):
    agent = AGiXT(agent_name=agent_name)
    response = await agent.run(
        task=prompt.prompt,
        prompt="instruct",
    )
    return {"response": str(response)}


@app.post("/api/agent/{agent_name}/smartinstruct/{shots}", tags=["Agent"])
async def smartinstruct(agent_name: str, shots: int, prompt: Prompt):
    agent = AGiXT(agent_name=agent_name)
    response = await agent.smart_instruct(task=prompt.prompt, shots=int(shots))
    return {"response": str(response)}


@app.post("/api/agent/{agent_name}/chat", tags=["Agent"])
async def chat(agent_name: str, prompt: Prompt):
    agent = AGiXT(agent_name=agent_name)
    response = await agent.run(task=prompt.prompt, prompt="Chat", context_results=6)
    return {"response": str(response)}


@app.post("/api/agent/{agent_name}/smartchat/{shots}", tags=["Agent"])
async def smartchat(agent_name: str, shots: int, prompt: Prompt):
    agent = AGiXT(agent_name=agent_name)
    response = await agent.smart_chat(task=prompt.prompt, shots=shots)
    return {"response": str(response)}


@app.get("/api/agent/{agent_name}/command", tags=["Agent"])
async def get_commands(agent_name: str):
    agent = Agent(agent_name=agent_name)
    return {"commands": agent.agent_config["commands"]}


@app.patch("/api/agent/{agent_name}/command", tags=["Agent"])
async def toggle_command(
    agent_name: str, payload: ToggleCommandPayload
) -> ResponseMessage:
    agent = Agent(agent_name=agent_name)
    print(payload)
    try:
        if payload.command_name == "*":
            for each_command_name in agent.agent_config["commands"]:
                agent.agent_config["commands"][each_command_name] = payload.enable

            agent.update_agent_config(
                new_config=agent.agent_config["commands"], config_key="commands"
            )
            return ResponseMessage(
                message=f"All commands enabled for agent '{agent_name}'."
            )
        else:
            agent.agent_config["commands"][payload.command_name] = payload.enable
            agent.update_agent_config(
                new_config=agent.agent_config["commands"], config_key="commands"
            )
            return ResponseMessage(
                message=f"Command '{payload.command_name}' toggled for agent '{agent_name}'."
            )
    except Exception as e:
        logging.info(e)
        raise HTTPException(
            status_code=500,
            detail=f"Error enabling all commands for agent '{agent_name}': {str(e)}",
        )


@app.post("/api/agent/{agent_name}/task", tags=["Agent"])
async def start_task_agent(agent_name: str, objective: Objective) -> ResponseMessage:
    task = Tasks(agent_name=agent_name)
    # If it's running stop it.
    task_status = task.get_status()
    if task_status != False:
        task.stop_tasks()
        return ResponseMessage(message="Task agent stopped")
    # If it's not running start it.
    try:
        asyncio.create_task(task.run_task(objective=objective.objective))
        return ResponseMessage(message="Task agent started")
    except Exception as e:
        raise HTTPException(
            status_code=500, detail="Error occurred while starting the task"
        )


# Get tasks Tasks(agent_name=agent_name).get_tasks()
@app.get("/api/agent/{agent_name}/tasks", tags=["Agent"])
async def get_tasks(agent_name: str) -> Dict[str, List[str]]:
    tasks = Tasks(agent_name=agent_name).get_tasks()
    return {"tasks": tasks}


@app.get("/api/agent/{agent_name}/task", tags=["Agent"])
async def get_task_output(agent_name: str) -> TaskOutput:
    try:
        task_output = Tasks(agent_name=agent_name).get_task_output()
    except:
        task_output = False
    if task_output != False:
        return TaskOutput(
            output=task_output,
            message="Task agent is not running",
        )
    else:
        return TaskOutput(
            output="",
            message="Task agent is not running",
        )


@app.get("/api/agent/{agent_name}/task/status", tags=["Agent"])
async def get_task_status(agent_name: str):
    task_status = Tasks(agent_name=agent_name).get_status()
    return {"status": task_status}


@app.get("/api/chain", tags=["Chain"])
async def get_chains():
    chains = Chain().get_chains()
    return chains


@app.get("/api/chain/{chain_name}", tags=["Chain"])
async def get_chain(chain_name: str):
    try:
        chain_data = Chain().get_chain(chain_name=chain_name)
        return {"chain": chain_data}
    except:
        raise HTTPException(status_code=404, detail="Chain not found")


@app.get("/api/chain/{chain_name}/responses", tags=["Chain"])
async def get_chain(chain_name: str):
    try:
        chain_data = Chain().get_step_response(chain_name=chain_name, step_number="all")
        return {"chain": chain_data}
    except:
        raise HTTPException(status_code=404, detail="Chain not found")


@app.post("/api/chain/{chain_name}/run", tags=["Chain"])
async def run_chain(chain_name: str) -> ResponseMessage:
    await Chain().run_chain(chain_name=chain_name)
    return {"message": f"Chain '{chain_name}' completed."}


@app.post("/api/chain", tags=["Chain"])
async def add_chain(chain_name: ChainName) -> ResponseMessage:
    Chain().add_chain(chain_name=chain_name.chain_name)
    return ResponseMessage(message=f"Chain '{chain_name.chain_name}' created.")


@app.put("/api/chain/{chain_name}", tags=["Chain"])
async def rename_chain(chain_name: str, new_name: ChainNewName) -> ResponseMessage:
    Chain().rename_chain(chain_name=chain_name, new_name=new_name.new_name)
    return ResponseMessage(
        message=f"Chain '{chain_name}' renamed to '{new_name.new_name}'."
    )


@app.delete("/api/chain/{chain_name}", tags=["Chain"])
async def delete_chain(chain_name: str) -> ResponseMessage:
    Chain().delete_chain(chain_name=chain_name)
    return ResponseMessage(message=f"Chain '{chain_name}' deleted.")


@app.post("/api/chain/{chain_name}/step", tags=["Chain"])
async def add_step(chain_name: str, step_info: StepInfo) -> ResponseMessage:
    Chain().add_chain_step(
        chain_name=chain_name,
        step_number=step_info.step_number,
        prompt_type=step_info.prompt_type,
        prompt=step_info.prompt,
        agent_name=step_info.agent_name,
    )
    return {"message": f"Step {step_info.step_number} added to chain '{chain_name}'."}


@app.put("/api/chain/{chain_name}/step/{step_number}", tags=["Chain"])
async def update_step(
    chain_name: str, step_number: int, chain_step: ChainStep
) -> ResponseMessage:
    Chain().update_step(
        chain_name=chain_name,
        step_number=chain_step.step_number,
        prompt_type=chain_step.prompt_type,
        prompt=chain_step.prompt,
        agent_name=chain_step.agent_name,
    )
    return {
        "message": f"Step {chain_step.step_number} updated for chain '{chain_name}'."
    }


@app.patch("/api/chain/{chain_name}/step/move", tags=["Chain"])
async def move_step(
    chain_name: str, chain_step_new_info: ChainStepNewInfo
) -> ResponseMessage:
    Chain().move_step(
        chain_name=chain_name,
        current_step_number=chain_step_new_info.old_step_number,
        new_step_number=chain_step_new_info.new_step_number,
    )
    return {
        "message": f"Step {chain_step_new_info.old_step_number} moved to {chain_step_new_info.new_step_number} in chain '{chain_name}'."
    }


@app.delete("/api/chain/{chain_name}/step/{step_number}", tags=["Chain"])
async def delete_step(chain_name: str, step_number: int) -> ResponseMessage:
    Chain().delete_step(chain_name=chain_name, step_number=step_number)
    return {"message": f"Step {step_number} deleted from chain '{chain_name}'."}


@app.post("/api/prompt", tags=["Prompt"])
async def add_prompt(prompt: CustomPromptModel) -> ResponseMessage:
    try:
        Prompts().add_prompt(prompt_name=prompt.prompt_name, prompt=prompt.prompt)
        return ResponseMessage(message=f"Prompt '{prompt.prompt_name}' added.")
    except Exception as e:
        raise HTTPException(status_code=400, detail=str(e))


@app.get("/api/prompt/{prompt_name}", tags=["Prompt"], response_model=CustomPromptModel)
async def get_prompt(prompt_name: str):
    try:
        prompt_content = Prompts().get_prompt(prompt_name=prompt_name)
        return {"prompt_name": prompt_name, "prompt": prompt_content}
    except Exception as e:
        raise HTTPException(status_code=404, detail=str(e))


@app.get("/api/prompt", response_model=PromptList, tags=["Prompt"])
async def get_prompts():
    prompts = Prompts().get_prompts()
    return {"prompts": prompts}


@app.delete("/api/prompt/{prompt_name}", tags=["Prompt"])
async def delete_prompt(prompt_name: str) -> ResponseMessage:
    try:
        Prompts().delete_prompt(prompt_name=prompt_name)
        return ResponseMessage(message=f"Prompt '{prompt_name}' deleted.")
    except Exception as e:
        raise HTTPException(status_code=404, detail=str(e))


@app.put("/api/prompt/{prompt_name}", tags=["Prompt"])
async def update_prompt(prompt: CustomPromptModel) -> ResponseMessage:
    try:
        Prompts().update_prompt(prompt_name=prompt.prompt_name, prompt=prompt.prompt)
        return ResponseMessage(message=f"Prompt '{prompt.prompt_name}' updated.")
    except Exception as e:
        raise HTTPException(status_code=404, detail=str(e))


@app.get("/api/prompt/{prompt_name}/args", tags=["Prompt"])
async def get_prompt_arg(prompt_name: str):
    return {"prompt_args": Prompts().get_prompt_args(prompt_name)}


@app.get("/api/extensions/settings", tags=["Extensions"])
async def get_extension_settings():
    try:
        return {"extension_settings": Extensions().get_extension_settings()}
    except Exception:
        raise HTTPException(status_code=400, detail="Unable to retrieve settings.")


@app.get("/api/extensions/{command_name}/args", tags=["Extension"])
async def get_command_args(command_name: str):
    return {"command_args": Extensions().get_command_args(command_name=command_name)}


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("--workers", type=int, default=None)
    args = parser.parse_args()
    uvicorn.run(app, host="127.0.0.1", port=7437, workers=args.workers)
